@@ -75,10 +75,16 @@ def md5_digest(data):
 if platform.python_implementation() == 'PyPy':
     def create_readline_wrapper(fh):
         fh.recv = fh.read
-        if not hasattr(fh, '_drop'):
-            fh._drop = lambda: None
-            fh._reuse = lambda: None
-        return socket._fileobject(fh, close=True)
+        if is_py2:
+            if not hasattr(fh, '_drop'):
+                fh._drop = lambda: None
+                fh._reuse = lambda: None
+            ans = socket._fileobject(fh, close=True)
+        else:
+            fh.recv_into = fh.readinto
+            fh._decref_socketios = lambda: None
+            ans = BufferedReader(socket.SocketIO(fh, 'r'))
+        return ans
 else:
     def create_readline_wrapper(fh):
         fh.recv = fh.read
@@ -595,8 +601,8 @@ class HTTPRedirectHandler(BaseHandler):
         """
         from ._request import Request
         m = req.get_method()
-        if (code in (301, 302, 303, 307, "refresh") and m in ("GET", "HEAD")
-                or code in (301, 302, 303, "refresh") and m == "POST"):
+        if (code in (301, 302, 303, 307, 308, "refresh") and m in ("GET", "HEAD")
+                or code in (301, 302, 303, 307, 308, "refresh") and m == "POST"):
             # Strictly (according to RFC 2616), 301 or 302 in response
             # to a POST MUST NOT cause a redirection without confirmation
             # from the user (of urllib2, in this case).  In practice,
@@ -609,7 +615,7 @@ class HTTPRedirectHandler(BaseHandler):
                 headers=req.headers,
                 origin_req_host=req.get_origin_req_host(),
                 unverifiable=True,
-                visit=False,
+                visit=False, method=m if code in (307, 308) else None,
                 timeout=req.timeout)
             new._origin_req = getattr(req, "_origin_req", req)
             return new
@@ -655,7 +661,7 @@ class HTTPRedirectHandler(BaseHandler):
         return self.parent.open(new)
 
     http_error_301 = http_error_303 = http_error_307 = http_error_302
-    http_error_refresh = http_error_302
+    http_error_refresh = http_error_308 = http_error_302
 
     inf_msg = "The HTTP server returned a redirect error that would " \
               "lead to an infinite loop.\n" \
@@ -1272,14 +1278,21 @@ class HTTPSHandler(AbstractHTTPHandler):
         if self.client_cert_manager is not None:
             key_file, cert_file = self.client_cert_manager.find_key_cert(
                 req.get_full_url())
-        if self.ssl_context is None:
-            conn_factory = partial(
-                HTTPSConnection, key_file=key_file,
-                cert_file=cert_file)
+        if sys.version_info > (3, 5):
+            import ssl
+            ctx = self.ssl_context or ssl.create_default_context()
+            if cert_file:
+                ctx.load_cert_chain(cert_file, key_file)
+            conn_factory = partial(HTTPSConnection, context=ctx)
         else:
-            conn_factory = partial(
-                HTTPSConnection, key_file=key_file,
-                cert_file=cert_file, context=self.ssl_context)
+            if self.ssl_context is None:
+                conn_factory = partial(
+                    HTTPSConnection, key_file=key_file,
+                    cert_file=cert_file)
+            else:
+                conn_factory = partial(
+                    HTTPSConnection, key_file=key_file,
+                    cert_file=cert_file, context=self.ssl_context)
         return self.do_open(conn_factory, req)
 
     https_request = AbstractHTTPHandler.do_request_
